#!/usr/bin/env python3
"""
MTR (My TraceRoute) Implementation
Parameters:
    - IP: Target IP address
Additional parameters in comments:
    - count: Number of pings per hop
Requires: scapy package and admin/root privileges
"""
from scapy.all import *
import time
import statistics
import argparse
from collections import defaultdict

class MTRTrace:
    def __init__(self, target_ip, count=10):
        self.target_ip = target_ip
        self.count = count
        self.results = defaultdict(lambda: {'sent': 0, 'received': 0, 'rtts': []})
        
    def send_probe(self, ttl):
        """Send a single probe with specified TTL."""
        try:
            ans, unans = sr1(IP(dst=self.target_ip, ttl=ttl)/ICMP(), timeout=2, verbose=0)
            if ans:
                return ans.src, ans.time - time.time()
            return None, None
        except Exception as e:
            print(f"Error sending probe: {e}")
            return None, None
    
    def trace(self):
        """Perform MTR trace to target."""
        print("\nStart MTR trace to", self.target_ip)
        print("TTL  Loss%  Snt  Last  Avg  Best  Wrst  StDev")
        
        for ttl in range(1, 31):  # Max 30 hops
            hop_stats = self.results[ttl]
            
            for _ in range(self.count):
                hop_stats['sent'] += 1
                ip, rtt = self.send_probe(ttl)
                if ip:
                    hop_stats['received'] += 1
                    hop_stats['rtts'].append(rtt * 1000)  # Convert to ms
            
            if hop_stats['received'] > 0:
                loss = 100 - (hop_stats['received'] / hop_stats['sent'] * 100)
                rtts = hop_stats['rtts']
                last = rtts[-1]
                avg = sum(rtts) / len(rtts)
                best = min(rtts)
                worst = max(rtts)
                stdev = statistics.stdev(rtts) if len(rtts) > 1 else 0
                
                print(f"{ttl:3d}  {loss:5.1f}  {hop_stats['sent']:4d}  {last:4.1f}  {avg:4.1f}  {best:4.1f}  {worst:4.1f}  {stdev:4.1f}")
                
                if ip == self.target_ip:
                    break
            else:
                print(f"{ttl:3d}  100.0  {self.count:4d}    *     *     *     *     *")
                
        print("\nTrace complete.")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="MTR (My TraceRoute) Implementation")
    parser.add_argument("ip", help="Target IP address")
    parser.add_argument("--count", type=int, default=10,
                      help="Number of pings per hop")
    args = parser.parse_args()
    
    mtr = MTRTrace(args.ip, args.count)
    mtr.trace()
